
# q-expansion of newform 351.2.t.b, downloaded from the LMFDB on 19 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 351
weight = 2
poly_data = [1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[326, [-1, 1]], [28, [-1, 0]]]
aps_data = [[0, 0], [0, 0], [2, 2], [-4, 2], [-4, 2], [-1, -3], [3, 0], [-2, 4], [3, -3], [0, 6], [2, 2], [-4, 8], [4, 4], [0, 1], [8, -4], [9, 0], [-2, -2], [0, -7], [0, 0], [0, 0], [6, -12], [0, -1], [-8, 4], [2, -4], [8, -4], [0, 3], [-8, 8], [-3, 0], [2, -4], [-15, 15], [16, 0], [15, -15], [24, -12], [-5, 5], [-4, -4], [-24, 12], [7, -7], [10, -20], [4, 4], [0, -21], [9, 0], [-7, 0], [0, 15], [0, 0], [-14, 28], [-17, 0], [-13, 13], [8, -4], [12, -6], [-2, -2], [-3, 0], [-6, -6], [16, -8], [21, 0], [15, -15], [0, -9], [-18, 0], [12, -24], [0, -26], [16, -8], [-7, 7], [-10, -10], [-6, 12], [0, 0], [0, 14], [28, -14], [28, -14], [-13, 13], [-3, 3], [-20, 10], [12, -6], [4, -8], [0, 25], [11, -11], [14, -28], [16, 16], [0, 33], [-16, 32], [-18, -18], [2, 2], [-9, 9], [-16, 8], [12, -24], [7, 0], [0, 7], [0, 15], [-14, 28], [28, -14], [-36, 18], [-8, -8], [15, 0], [-44, 22], [2, -4], [-36, 36], [-24, -24], [-21, 0], [16, 16], [-18, 0], [20, 0], [-4, 8], [0, -20], [-2, 4], [21, -21], [0, -6], [-4, 4], [6, -12], [48, -24], [2, -4], [39, -39], [0, -37], [13, -13], [-12, 24], [-6, -6], [-12, 6], [-28, 56], [0, -30], [24, 24], [3, 0], [-18, 18], [0, -36], [-12, -12], [0, -41], [0, 42], [26, -52], [-8, 4], [-9, 0], [28, -14], [36, 0], [0, 47], [-2, -2], [16, -32], [-6, -6], [4, -4], [-5, 0], [30, 30], [-4, -4], [-6, 12], [-12, -12], [-30, 30], [51, 0], [-20, 40], [-56, 28], [31, -31], [-14, 28], [-38, 0], [48, -24], [36, -18], [0, -18], [41, -41], [32, -64], [-10, -10], [21, 0], [16, 0], [57, -57], [0, -19], [0, 27], [31, 0], [36, -18], [7, 0], [12, 12], [28, -14], [-30, 0], [6, 6], [36, 0], [-14, -14], [12, -6], [5, 0], [0, 25]]
